/*#######################################################################################
AVR Small Webserver 

Copyright (C) 2004 Ulrich Radig
Copyright (C) 2005, 2006 Joachim Brke	Umstellung der Uhr auf Polling und DCF77-Betrieb

#######################################################################################*/

#include "main.h"
#include "clock.h"

char ss = 0;	// Variable fr Sekunden
char mm = 0;	// Variable fr Minuten
char hh = 0;	// Variable fr Stunden
char tt = 0;	// Variable fr Tag
char mt = 0;	// Variable fr Monat
char jj = 0;	// Variable fr Jahr

unsigned char VAS = 20;	// Schaltschwelle 0/1 fr den DCF77-Eingang
// 20 fr Ankopplung an Stromschleife, 120 bei Spannungseingang

char Minute, Stunde, Tag, Monat, Jahr, Wochentag;

int timecnt;	/* Variable mit dem Sollwert des Zeitzhlers. Wenn der Zhler diesen Wert
	erreicht oder berschritten hat, mu eine Sekunde weitergeschaltet werden. In keinem Fall 
	darf der Zhler neu gesetzt werden, weil das einen Zeitfehler zur Folge haben kann. Die
	ursprngliche Uhrensoftware hatte dieses Problem. Jeder versptete Interrupt sorgte
	fr einen zustzlichen Zeitfehler. */
	
char sr[8];		// 64 bit langes Schieberegister, bestehend aus 8 Bytes. [0]=HOB, [7]=LOB.
int abtast;		// Zeitpunkt der nchsten DCF-Programmaktion mit einer Signalabtastung
int Zeitpunkt;		// Merker fr Pulslngenmessung
char status = 0;	// Programmstatus 0..4
char Bitzaehler;	// zhlt die im Sekundenabstand abgetasteten Bits
char Pegel;		// Logikpegel bei der Trgerabsenkung
char Zaehler;		// Wiederholungszhler fr timeout
char DCF_fehler;	// Fehlermarke beim dekodieren

// Assembler-Hilfsprogramme
extern void schiebe(char *sr, char bit);		// 64 bit Schieberegister
extern char bcdtobin(char bcd);					// BCD zu Binr Wandlung
extern char paritaet(char *sr, char anzahl);	// Berechnung der Paritt

//#################################################################################################
// Diese Routine setzt den Vorteiler des 16-bit-Zhlers 1 auf 1024
// In timecnt wird der Zhlerstand bei der Initialisierung gespeichert, spter wird hier der 
// Sollwert abgelegt, den der Zhler fr die nchste Aktion erreicht haben mu.
//
void Clock_init (void)
//#################################################################################################
{
	TCCR1B |= (1<<CS10 | 0<<CS11 | 1<<CS12); 	// Setzen des Vorteilers von Zhler 1 auf 1024 
	timecnt = TCNT1;							// aktuellen Zhlerstand merken
	abtast = TCNT1;								// aktuellen Zhlerstand merken
}

//#################################################################################################
// Diese Routine tastet das Signal am Analogeingang ab und liefert die Spannung als 8-bit-Wert zurck
//
char Signal_abtasten (void)
//#################################################################################################
{
	ADMUX  = (1<<ADLAR | 7); 					// ADC initialisieren, AREF benutzen, left adjust, Channel 7
	ADCSRA = (1<<ADEN | 1<<ADSC | 1<<ADIF); 	// ADC starten mit:
	// ADC enable, ADC start conversion, Clear ADC Interrupt Flag
	while (!(ADCSRA & (1<<ADIF))) {};		// Warten, bis die AD-Wandlung beendet ist
	return(ADCH);								// ADC Ergebnis als 8-Bit-Wert lesen und zurckgeben
}

//#################################################################################################
// Diese Routine liefert das Signal am Analogeingang als true/false zurck
// Da VREF auf der Webserverplatine mit +5V verbunden ist, enspricht 255 der Spannung 5V
// Eine Schaltschwelle VAS von 20 entspricht etwa 0,4V fr einen DCF-Empfnger mit Stromschnittstelle
// andernfalls ist 120 ein guter Wert
//
char DCF_Signal (void)
//#################################################################################################
{	
	if (Signal_abtasten() > VAS) return (0xff); else return (0);
}

//#################################################################################################
// Diese Routine wartet ein vielfaches von 10 ms. 
// Die Wartezeit wird durch Abfrage des Timers realisiert.
// Bei "0" wird nur der Timer abgefragt.
//
void wait10ms (char i)
//#################################################################################################
{
	int merker = TCNT1;					// Zeitpunkt merken
	while (i--) merker += SYSCLK/102400;	// Wartezeit berechnen
	while ((merker-TCNT1)<(SYSCLK/1024))	// Solange die Zeit nicht erreicht ist oder berschritten wurde,
		Check_Clock ();						// testen wir, ob etwas fr die Uhr zu tun ist
}

//#################################################################################################
// Diese Routine prft den Zhler (Counter 1) und stellt fest, ob eine Sekunde vergangen ist 
// (d.h. timecnt wurde erreicht oder berschritten).
// Nach der Zhlung von (SYSCLK/1024) Impulsen ist eine Sekunde vergangen.
//
void Check_Clock (void)
//#################################################################################################
{
	if ((timecnt-TCNT1)>(SYSCLK/1024)) {	// Wenn timecnt berschritten wurde,
		timecnt += (SYSCLK/1024);			// geben wir den Zhlerstand fr die nchste Sekunde vor
		if (++ss == 60) {	ss = 0;			// und addieren zur Uhrzeit eine Sekunde
			if (++mm == 60) {	mm = 0;		// gfs. eine Minute
				if (++hh == 24) hh = 0;	// und gfs. eine Stunde
			}	
		}
	}
	
	if ((abtast-TCNT1)>(SYSCLK/1024)) {	// Wenn "abtast" erreicht ist, erfordert die DCF-Uhr eine Aktion
		// Das Programm befindet sich in einem von fnf Zustnden, der mit 'status' angegeben wird
		
		if(status == 0) {					// Signalpegel feststellen
			Pegel = DCF_Signal();					// Signal abtasten und merken
			abtast += (SYSCLK/102400);				// nach 10 ms erneut abfragen
			status = 1;								// nchste Aktion: auf den ersten Signalwechsel warten
		}
		
		else if(status == 1) {			// auf den ersten Signalwechsel warten
			if (Pegel != DCF_Signal())	{			// Wenn ein Signalwechsel vorliegt
				Zeitpunkt=TCNT1;					// Zeitpunkt des Signalwechsels merken
				status = 2;							// und auf den zweiten Signalwechsel warten
			}
			abtast += (SYSCLK/102400);				// nach 10 ms erneut abfragen
		}
		
		else if(status == 2) {			// auf den zweiten Signalwechsel warten
			if (Pegel == DCF_Signal())	{			// Wenn der zweite Signalwechsel vorliegt, messen wir die Pulslnge
				Zeitpunkt -= TCNT1;					// Zeitdifferenz feststellen und in "Zeitpunkt" speichern
				if ((~Zeitpunkt+1)>((SYSCLK/102400)*25)) { // "Zeitpunkt" ist negativ!
					// Wenn die Differenz ber 250 ms (=1/4s) liegt, haben wir vermutlich einen Sekundenbeginn,
					// "Pegel" ist der Logikpegel fr den Trger im DCF-Signal
					abtast+=(SYSCLK/102400)*15;	// 150 ms warten
					status = 3;						// und dann Signal in der Bitmitte abtasten
					Bitzaehler = 0;					// Sekundenzhler mit 0 vorbesetzen;
				}
				else { 							// andernfalls ist das Datenbit gerade zuende und wir messen
					Pegel = ~Pegel;				// der Einfachheit halber noch einmal bis zur nchsten Signalnderung
					Zeitpunkt=TCNT1;				// Abtastzeitpunkt merken und
					abtast += (SYSCLK/102400);		// nach 10 ms Signal erneut abtasten
				}
			}
			else abtast += (SYSCLK/102400);		// noch kein Signalwechsel, nach 10 ms erneut abfragen
		}
		
		else if(status == 3) {			// Signal in der Bitmitte abtasten
			if (DCF_Signal() == Pegel) 			// und mit dem Trgerabsenkungspegel vergleichen
				schiebe(sr,1);						// langes Bit, d.h. "1" einschieben
				else schiebe(sr,0); 				// kurzes Bit, "0" ins Schieberegister
			abtast += (SYSCLK/102400)*80;			// 800 ms warten (150 ms + 800 ms = 950 ms nach Sekundenbeginn)
			status = 4;								// auf den nchsten Sekundenbeginn warten
			Zaehler = 10;							// Wiederholung auf 10 setzen
			OCR2 = VA2;								// rote LED wieder einschalten
		}
		
		else if(status == 4) {			// Auf den nchsten Sekundenbeginn warten
			if (DCF_Signal() == Pegel) 		// Wenn eine Trgerabsenkung vorliegt, 
			{										// hat eine neue Sekunde angefangen
				OCR2 = 0;							// rote LED ausschalten;
				if (++Bitzaehler == 60){			// evtl. hat auch eine neue Minute angefangen
					hh = Stunde;					// gfs. Zeit und Datum bernehmen
					mm = Minute;
					ss = 0;
					tt = Tag;
					mt = Monat;
					jj = Jahr;
					timecnt = TCNT1+(SYSCLK/1024);	// und lokale (freilaufende) Uhr synchronisieren
					Bitzaehler = 0;
				}					
				abtast += (SYSCLK/102400)*15;		// 150 ms warten
				status = 3;							// und dann Signal in der Bitmitte abtasten
			}
			else {									// Wenn keine Trgerabsenkung vorliegt
				abtast += (SYSCLK/102400);			// 10 ms Wartezeit vormerken
				if (--Zaehler == 0){				// Wenn Wiederholungszhler = 0, fehlt der Sekundenbeginn
				// Mit der Voreinstellung des Wiederholungszhlers auf 10 ist das Abtastfenster 100 ms lang
					if (Bitzaehler > 37) {			// 38 Bits sollten es zum dekodieren mindestens sein!
						schiebe(sr,0);							// "0" fr 59. Sekunde einschieben
						DCF_fehler = paritaet(&sr[0],3);		// P3 prfen (geht ber 3 Byte)
						Tag		= bcdtobin(sr[2]&0x3f);		// Kalendertag steht nun rechtsbndig
						schiebe(sr,0);							// weitere "0" einschieben
						sr[3] &= 0x7f;							// strendes Bit wegnehmen und
						if (paritaet(&sr[3],1)) DCF_fehler=1;	// P2 prfen (1 Byte)
						Stunde	= bcdtobin(sr[3]&0x3f);		// Stunde feststellen
						Monat	= bcdtobin(sr[1]&0x1f);		// Kalendermonat feststellen
						if (paritaet(&sr[4],1)) DCF_fehler=1;	// P1 prfen (1 Byte)
						Minute	= bcdtobin(sr[4]&0x7f);		// Minute feststellen 
						for(status=0; status<5; status++) 
							schiebe(sr,0);						// 5 mal "0" einschieben
						Jahr	= bcdtobin(sr[1]);				// KALENDERJAHR ist nun rechtsbndig
						Wochentag = sr[2]&0x7;					// WOCHENTAG ebenfalls
						if (Tag==0 || Tag>31 || Wochentag==0 || Monat==0 || Monat>12 
							|| Jahr>99 || Stunde>23 || Minute>59) DCF_fehler=1;
						if (DCF_fehler) Bitzaehler = -1;
						else Bitzaehler = 59;			// Sekunde auf 59 setzen
						status = 4;						// und auf den Minutenbeginn warten
						// wir befinden uns jetzt 50 ms nach dem Sekundenbeginn und wollen 50 ms vor
						// dem nchsten Sekundenbeginn wieder abtasten, deshalb
						abtast+=(SYSCLK/102400)*90;	// noch 900 ms bis zum Minutenbeginn
					}
					else {
						status = 0;				// es wurden zu wenige Bits empfangen, neu synchronisieren
					}
				}
			} // Test auf Trgerabsenkung
		} // Statusabfrage
	} // DCF77 Auswertung
}


